/*
 *
 *  Kepas - KDE Easy Publish and Share
 *
 *  Copyright (C) 2008  Tom Patzig <tpatzig@suse.de>
 *
 *  This file is part of kepas.
 *
 *  kepas is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  kepas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with kepas; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
*/

#include "buddylist.h"


BuddyList::BuddyList(QObject* parent)
{

	m_parent = parent;
	setupUi(this);
	//	 m_config = config;
	setWindowIcon(KIcon("folder-remote"));
	
	count = 0;
	buddyVector = new QVector<BuddyContent*>;

	icqListWidget = 0;
	buddyListWidget = new QListWidget();
	
	buddyListWidget->setMouseTracking(true);
	buddyListWidget->setSelectionRectVisible(false);
	buddyListWidget->setAutoFillBackground(false);
	buddyListWidget->setViewMode(QListView::IconMode);
	buddyListWidget->setResizeMode(QListView::Adjust);
	buddyListWidget->setEnabled(true);

	tabWidget->clear();	
	tabWidget->addTab(buddyListWidget, SmallIcon("network-workgroup"), i18n("LAN"));
	
	buddyItemOld = 0;
	dbus = 0;
	
	connect(buddyListWidget, SIGNAL(itemEntered(QListWidgetItem*)),this,SLOT(slotItemEntered(QListWidgetItem*)));
//	connect(buddyListWidget, SIGNAL(itemSelectionChanged()),this,SLOT(slotItemSelectionChanged()));

//	checkICQStatus();
	
}


BuddyList::~BuddyList()
{
	qDebug() << "deleting buddylist";

	if (buddyListWidget)
		delete buddyListWidget;

	if (icqListWidget)
		delete icqListWidget;

	if (buddyVector)
		delete buddyVector;

	if (dbus)
		delete dbus;

	if (buddyItemOld)
		delete buddyItemOld;
}

void BuddyList::initDefaultBuddyList(QString name)
{	
	checkICQStatus();	

	buddyListWidget->setEnabled(true);
	connect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotPopupMenu(QListWidgetItem*)));

	if (icqListWidget != 0) {
		connect(icqListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotPopupKopeteMenu(QListWidgetItem*)));
	}

	m_userName = name;
	show();
}


void BuddyList::initSendClipBuddyList(QString clipEntry, QString username)
{
	disconnect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotPopupMenu(QListWidgetItem*)));
	disconnect(icqListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotPopupKopeteMenu(QListWidgetItem*)));

	buddyListWidget->setEnabled(true);
	m_clipEntry = clipEntry;
	connect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendClip(QListWidgetItem*)));
	m_userName = username;
	show();
}


void BuddyList::initSendFileBuddyList(QString file, QString username)
{
	checkICQStatus();	
	disconnect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotPopupMenu(QListWidgetItem*)));
        disconnect(icqListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotPopupKopeteMenu(QListWidgetItem*)));

	buddyListWidget->setEnabled(true);
	qDebug() << "initSendFileBuddyList";
	m_fileName = file;
	connect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendFileDirect(QListWidgetItem*)));

	if (icqListWidget != 0) {
		connect(icqListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendFileDirectICQ(QListWidgetItem*)));
	}
	m_userName = username;
	show();
}



void BuddyList::addService(QMap<QString, QByteArray> info)
{
	count++;

	QString username = info["User Name"];
	QString hostname = info["Machine Name"];
	QString port = info["port"];
	
	if (!buddyVector->isEmpty())
		for (int i = 0; i < buddyVector->count(); i++) {
			tmpBuddyContent = buddyVector->at(i);
			if (tmpBuddyContent->name == username && tmpBuddyContent->hostname == hostname)
				return;

		}
	newBuddyContent = new BuddyContent(username,hostname,port.toInt(),info["SN"],this);
	buddyVector->push_back(newBuddyContent);

	buddy = new QListWidgetItem(KIcon("user"), QString(username + "@" + hostname),buddyListWidget);
//	buddy = new QListWidgetItem(KIcon("user"), QString(username + "@" + hostname + ":" + port),buddyListWidget);
	buddyListWidget->addItem(buddy);


}

void BuddyList::delService(QString sn)
{

	qDebug() << "Deleting Buddy!";
	if (!buddyVector->isEmpty())
		for (int i = 0; i < buddyVector->count(); i++) {
			tmpBuddyContent = buddyVector->at(i);
	//		qDebug() << "Name:" << tmpBuddyContent->name << " host: " << tmpBuddyContent->hostname;

			if (tmpBuddyContent->SN == sn) {
				qDebug() << "take item at pos: " << i;
				buddyListWidget->takeItem(i);
				buddyVector->remove(i);
			}
		}
}


void BuddyList::slotItemEntered(QListWidgetItem* buddyItem)
{
	 if (buddyItemOld != 0) {
	 	 buddyItemOld->setBackground(Qt::white);
	 	 buddyItemOld->setForeground(Qt::black);
	 }
	 buddyItem->setBackground(QColor("aliceblue"));
	 buddyItem->setForeground(Qt::red);
	 buddyItemOld = buddyItem;

}

void BuddyList::slotItemSelectionChanged()
{
	qDebug() << "changed";



}

void BuddyList::slotPopupMenu(QListWidgetItem* buddyItem)
{

         buddyItem->setBackground(QColor("aliceblue"));
         buddyItem->setForeground(Qt::red);

         popupMenu = new QMenu(this);
         popupMenu->setPalette(Qt::white);

         sendFileAction = popupMenu->addAction(KIcon("text-directory"), "Send File...");
         connect(sendFileAction, SIGNAL(triggered(bool)), this, SLOT(slotSendFile(bool)));
/*
         sendDirAction = popupMenu->addAction(KIcon("inode-directory"), "Send Directory...");
         connect(sendDirAction, SIGNAL(triggered(bool)), this, SLOT(slotSendDir(bool)));
*/
	 sendClipMenu = popupMenu->addMenu(KIcon("klipper"),"Send Clipboard");
	 addClipEntries();

//	 getClipAction = popupMenu->addAction(KIcon("klipper"), "Get Clipboard");
//       connect(getClipAction, SIGNAL(triggered(bool)), this, SLOT(slotGetClip(bool)));

         popupMenu->exec(QCursor::pos());
//       pictureButton->setMenu(popupMenu);

}

void BuddyList::slotPopupKopeteMenu(QListWidgetItem* buddyItem)
{

         buddyItem->setBackground(QColor("aliceblue"));
         buddyItem->setForeground(Qt::red);

         popupMenu = new QMenu(this);
         popupMenu->setPalette(Qt::white);

         sendFileAction = popupMenu->addAction(KIcon("text-directory"), "Send File...");
         connect(sendFileAction, SIGNAL(triggered(bool)), this, SLOT(slotSendFile(bool)));
/*
         sendDirAction = popupMenu->addAction(KIcon("inode-directory"), "Send Directory...");
         connect(sendDirAction, SIGNAL(triggered(bool)), this, SLOT(slotSendDir(bool)));
*/
         popupMenu->exec(QCursor::pos());

}

void BuddyList::slotSendFile(bool ok)
{

	 QListWidgetItem* current = buddyListWidget->selectedItems().at(0);
 	 current->setBackground(QColor("aliceblue"));
     	 current->setForeground(Qt::red);

	 int currentIndex = buddyListWidget->row(current);
	 BuddyContent* cBC = buddyVector->at(currentIndex);
	 qDebug() << "current Buddy Content: user " << cBC->name;
	 qDebug() << "current Buddy Content: SN " << cBC->SN;
	
	 QString fileName = QFileDialog::getOpenFileName(this, tr("Select File"),"./","*");
	 sendHandler = new Sender(m_parent,cBC->hostname,cBC->port,m_userName);
	 sendHandler->initFileSender(fileName,1);
//	 connect(sendHandler, SIGNAL(requestFinished(int)), this, SLOT(slotRequestSend(int)));
//	 connect(sendHandler, SIGNAL(requestFailed(QString)), this, SLOT(slotRequestFailed(QString)));
	 sendHandler->sendFileRequest();

}

void BuddyList::slotSendClip(QListWidgetItem* buddyItem)
{
	 disconnect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendClip(QListWidgetItem*)));

	 buddyItem->setBackground(QColor("aliceblue"));
     	 buddyItem->setForeground(Qt::red);
	 buddyListWidget->setEnabled(false);
	 if (icqListWidget != 0)
		icqListWidget->setEnabled(false);

	 int currentIndex = buddyListWidget->row(buddyItem);
	 BuddyContent* cBC = buddyVector->at(currentIndex);

	 sendHandler = new Sender(m_parent,cBC->hostname,cBC->port,m_userName);
	 sendHandler->initClipSender();
	 sendHandler->sendClipRequest(m_clipEntry);
	
	 connect(sendHandler,SIGNAL(transferFinished()), this, SLOT(slotSendFinished()));
	 connect(sendHandler,SIGNAL(transferDenied()), this, SLOT(slotSendFinished()));

}


void BuddyList::slotSendFileDirect(QListWidgetItem* buddyItem)
{

	 disconnect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendFileDirect(QListWidgetItem*)));
	 buddyItem->setBackground(QColor("aliceblue"));
     	 buddyItem->setForeground(Qt::red);
	 buddyListWidget->setEnabled(false);
	 if (icqListWidget != 0)
		icqListWidget->setEnabled(false);

	 int currentIndex = buddyListWidget->row(buddyItem);
	 BuddyContent* cBC = buddyVector->at(currentIndex);

	 sendHandler = new Sender(m_parent,cBC->hostname,cBC->port,cBC->name);
	 sendHandler->initFileSender(m_fileName,1);
	 sendHandler->sendFileRequest();
	
	 connect(sendHandler,SIGNAL(transferFinished()), this, SLOT(slotSendFinished())); 
	 connect(sendHandler,SIGNAL(transferDenied()), this, SLOT(slotSendFinished()));

}

void BuddyList::slotSendFileDirectICQ(QListWidgetItem* buddyItem)
{
	
	disconnect(icqListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendFileDirectICQ(QListWidgetItem*)));
	buddyItem->setBackground(QColor("aliceblue"));
        buddyItem->setForeground(Qt::red);
	buddyListWidget->setEnabled(false);
	if (icqListWidget != 0)
		icqListWidget->setEnabled(false);

        QString displayName = buddyItem->text();
	
	dbus = new QDBusConnection("dbus");
        QDBusConnection dbusconn = dbus->connectToBus(QDBusConnection::SessionBus, "dbus");
        QString service = "org.kde.kopete";
        QString path = "/Kopete";
        QString method = "sendFile";
        QString service2 = "org.kde.Kopete";

        QDBusInterface* interface = new QDBusInterface(service,path,service2,dbusconn);
        interface->call(method,displayName,m_fileName);
	
}



void BuddyList::slotSendDir(bool ok)
{

}


void BuddyList::slotGetClip(bool ok)
{
/*
	QListWidgetItem* current = buddyListWidget->selectedItems().at(0);
	int currentIndex = buddyListWidget->row(current);
        BuddyContent* cBC = buddyVector->at(currentIndex);

	getClipHandler = new RequestClipboard(this,cBC->hostname,cBC->port,cBC->name);
	getClipHandler->sendRequest();
*/

}

void BuddyList::addClipEntries()
{

	sendClipMenu->clear();
	dbus = new QDBusConnection("dbus");
	QDBusConnection dbusconn = dbus->connectToBus(QDBusConnection::SessionBus, "dbus");
	QString service = "org.kde.klipper";
	QString path = "/klipper";
	QString method = "getClipboardHistoryMenu";
	QString service2 = "org.kde.klipper.klipper";

	QDBusInterface* interface = new QDBusInterface(service,path,service2,dbusconn);
	QDBusReply<QStringList> reply = interface->call(method);
	
//	qDebug() << "klipper reply: " << reply.value();
	QStringList clipList = reply.value();

	for (int i=0; i < clipList.size(); i++){
		if (clipList.at(i).size() >= 30) {
			QString cut = clipList.at(i);
			cut.truncate(30);
			
			clipAction = sendClipMenu->addAction(cut + " ...");
	         } else
			clipAction = sendClipMenu->addAction(clipList.at(i));
		
		clipAction->setToolTip(clipList.at(i));
		
	}

 	connect(sendClipMenu, SIGNAL(triggered(QAction*)), this, SLOT(slotSendClipEntry(QAction*)));

}

void BuddyList::slotSendClipEntry(QAction* action)
{
	QListWidgetItem* current = buddyListWidget->selectedItems().at(0);
	slotSendClip(current);

}

void BuddyList::slotSendFinished()
{
	disconnect(sendHandler,SIGNAL(transferFinished()), this, SLOT(slotSendFinished()));
	disconnect(sendHandler,SIGNAL(transferDenied()), this, SLOT(slotSendFinished()));
	disconnect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendFileDirect(QListWidgetItem*)));
	disconnect(buddyListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendClip(QListWidgetItem*)));
	if (icqListWidget != 0) {
		disconnect(icqListWidget, SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(slotSendFileDirectICQ(QListWidgetItem*)));
	}
	sendHandler->deleteLater();
	qDebug() << "sendHandler deleted";
	
	close();

}


void BuddyList::checkICQStatus()
{
	if (icqListWidget)
		tabWidget->removeTab(1);

	dbus = new QDBusConnection("dbus");
        QDBusConnection dbusconn = dbus->connectToBus(QDBusConnection::SessionBus, "dbus");
        QString service = "org.kde.kopete";
        QString path = "/Kopete";
        QString methodAccount = "accounts";
	QString isConnected = "isConnected";
        QString service2 = "org.kde.Kopete";

        QDBusInterface* interface = new QDBusInterface(service,path,service2,dbusconn);
        QDBusReply<QStringList> reply = interface->call(methodAccount);
	if (reply.isValid()) {	
		QStringList accountList = reply.value();

		for (int i = 0; i < accountList.size(); i++)
		{
			QDBusReply<bool> reply1 = interface->call(isConnected,"ICQ",accountList.at(i));
			if (reply1.value() == true) {
				icqID = accountList.at(i);
				icqListWidget = new QListWidget();
				tabWidget->addTab(icqListWidget, SmallIcon("licq"),"Kopete-ICQ");
				fillIcqTab();
				break;
			}
		}
		
		qDebug() << "ICQ ID = "  << icqID;
	}
}


void BuddyList::fillIcqTab()
{

	icqListWidget->clear();
	
	dbus = new QDBusConnection("dbus");
        QDBusConnection dbusconn = dbus->connectToBus(QDBusConnection::SessionBus, "dbus");
        QString service = "org.kde.kopete";
        QString path = "/Kopete";
        QString contacts = "contacts";
        QString service2 = "org.kde.Kopete";

        QDBusInterface* interface = new QDBusInterface(service,path,service2,dbusconn);
        QDBusReply<QStringList> reply = interface->call(contacts);
	
        QStringList contactList = reply.value();

	icqListWidget->setMouseTracking(true);
	icqListWidget->setSelectionRectVisible(false);
	icqListWidget->setAutoFillBackground(false);
	icqListWidget->setViewMode(QListView::IconMode);
	icqListWidget->setResizeMode(QListView::Adjust);
	icqListWidget->setEnabled(true);

	for (int i = 0; i < contactList.size(); i++)
	{
		QStringList contactInfo = contactList.at(i).split(":");
		if (contactInfo.at(0) == "ICQProtocol") {
			QString contactID = contactInfo.at(2);
			QString contactName = getContactName(contactID);
			if (isOnline(contactName)) {
				buddy = new QListWidgetItem(KIcon("user"), QString(contactName),icqListWidget);
				icqListWidget->addItem(buddy);
			}
		}
	}

	connect(icqListWidget, SIGNAL(itemEntered(QListWidgetItem*)),this,SLOT(slotItemEntered(QListWidgetItem*)));
}


QString BuddyList::getContactName(QString id)
{

	dbus = new QDBusConnection("dbus");
        QDBusConnection dbusconn = dbus->connectToBus(QDBusConnection::SessionBus, "dbus");
        QString service = "org.kde.kopete";
        QString path = "/Kopete";
        QString method = "getDisplayName";
        QString service2 = "org.kde.Kopete";

        QDBusInterface* interface = new QDBusInterface(service,path,service2,dbusconn);
        QDBusReply<QString> reply = interface->call(method,id);
	
        QString name = reply.value();
	
	return name;
}

bool BuddyList::isOnline(QString name)
{

	dbus = new QDBusConnection("dbus");
        QDBusConnection dbusconn = dbus->connectToBus(QDBusConnection::SessionBus, "dbus");
        QString service = "org.kde.kopete";
        QString path = "/Kopete";
        QString method = "isContactOnline";
        QString service2 = "org.kde.Kopete";

        QDBusInterface* interface = new QDBusInterface(service,path,service2,dbusconn);
        QDBusReply<bool> reply = interface->call(method,name);
	
        bool status = reply.value();
	
	return status;
}

